''*****************************************************************************
''
''	Platformer tutorial 101
''
''	Chapter 03-A
''
''  Tiled Map Collision
''
''	Relminator (Richard Eric M. Lope)
''	http://rel.phatcode.net
''
''
''  I'm using EASY GL2D for rendering.
'' 	So try to read these tutorials for Easy GL2D..
'' 	http://back2basic.phatcode.net/?Issue_%232:Basic_2D_Rendering_in_OpenGL_using_Easy_GL2D%3A_Part_1
'' 	http://back2basic.phatcode.net/?Issue_%232:Basic_2D_Rendering_in_OpenGL_using_Easy_GL2D%3A_Part_2
''
''*****************************************************************************

#include once "fbgfx.bi"
#include once "/gl/gl.bi" 
#include once "/gl/glu.bi"   

#include once "FBGL2D.bi"     	'' We're gonna use Hardware acceleration
#include once "FBGL2D.bas"		'' So we'll be using my LIB


declare sub ConvertMap( Map() as integer, StrMap() as string )
declare sub DrawMap( byval PlayerX as single, byval PlayerY as single, Map() as integer )

''*****************************************************************************
const as integer SCREEN_WIDTH = 640
const as integer SCREEN_HEIGHT = 480

const as integer TILE_SIZE = 32

const as integer FALSE = 0
const as integer TRUE = not FALSE

const as single GRAVITY = 0.40
const as single JUMPHEIGHT = 7



''*****************************************************************************
''
''	Our Player Class
''
''*****************************************************************************
Type Player

public:
	
	'' Player States for Finite State Machine
	Enum E_STATE
		IDLE = 0,
		WALKING,
		JUMPING,
		FALLING,
		BORED,
		PLANT_BOMB,
		LIGHT_DYNAMITE,
		DIED
	End Enum
	
	'' Direction our player is facing for flipmode
	Enum E_DIRECTION
		DIR_RIGHT = 0,
		DIR_LEFT,
		DIR_UP,
		DIR_DOWN	
	End Enum
	
	'' Collision status returned when we collide with map
	Enum E_COLLISION
		COLLIDE_NONE = 0,
		COLLIDE_RIGHT,
		COLLIDE_LEFT,
		COLLIDE_CEILING,
		COLLIDE_FLOOR	
	End Enum
	
	'' length of time before the player gets "bored" standing up 
	Enum 
		BORED_WAIT_TIME = 60 * 3
	End Enum
	
	declare constructor()
	declare destructor()
	
	declare Sub Update( Map() as integer )			'' This updates the player depending on its State
	declare sub Draw(SpriteSet() as GL2D.IMAGE ptr)				'' Draws the Player according to state
	
	declare property GetX() as Single
	declare property GetY() as Single

private:
	
	declare Sub UpdateIdle( Map() as integer )			''\ 
	declare Sub UpdateWalking( Map() as integer )		'' |  
	declare Sub UpdateJumping( Map() as integer )		'' | 
	declare Sub UpdateBored( Map() as integer )			''  \ These are the functions to be called by Update
	declare Sub UpdateFalling( Map() as integer )		''  / Depending on the Player.State
	declare Sub UpdateLightDynamite( Map() as integer )	'' |
	declare Sub UpdatePlantBomb( Map() as integer )		'' |
	declare Sub UpdateDied( Map() as integer )			''/
	
	declare sub ResolveAnimationParameters()	'' Sets up animation params depending on State
	declare sub Animate()						'' Animates the player

	declare function CollideWalls(byval ix as integer, byval iy as integer, byref TileX as integer, Map() as integer ) as integer
	declare function CollideFloors(byval ix as integer, byval iy as integer, byref TileY as integer, Map() as integer ) as integer
	declare function CollideOnMap( Map() as integer ) as integer

	x		as single				'' Position
	y		as single
	Dx		as single				'' Direction
	Dy		as Single
	Speed	as single				'' Horizontal Speed
	Wid		as integer				'' Width of the player
	Hei		as integer				
	CanJump as integer				'' If the player can Jump
	
	Counter 	as Integer
	Frame		as Integer
	BaseFrame 	as Integer
	MaxFrame	as Integer	
	FlipMode	as Integer
	
	Direction 		as Integer
	StandingCounter as Integer
	
	State 			as E_STATE				'' State of the player
	
End Type

''*****************************************************************************
''
''*****************************************************************************
constructor Player()
	
	Counter = 0
	x = 32 * 2 
	y = 32 * 6 
	Speed = 2.5
	Wid = 24
	Hei = 30
	CanJump = false
	
	Frame = 0
	BaseFrame = 0
	MaxFrame = 8	
	FlipMode = GL2D.FLIP_NONE
	
	Direction = DIR_RIGHT
	
	State = FALLING
	StandingCounter = 0
	ResolveAnimationParameters()
	
End Constructor

''*****************************************************************************
''
''*****************************************************************************
destructor Player()

End Destructor

''*****************************************************************************
'' Collides the player box with the tiles on the x-axis
''*****************************************************************************
function Player.CollideWalls(byval ix as integer, byval iy as integer, byref TileX as integer, Map() as integer ) as integer
	
	dim as integer TileYpixels = iy - (iy mod TILE_SIZE)   '' Pixel of the player's head snapped to map grid
	dim as integer TestEnd = iy + hei					   '' Foot of the player
	
	TileX = ix\TILE_SIZE								   '' Current X map coord the player is on + x-velocity(+ width when moving right)
	
	dim as integer TileY = TileYpixels\TILE_SIZE		   '' Current Y map coord of the player's head
	
	'' Scan downwards from head to foot if we collided with a tile on the right or left
	while( TileYpixels <= TestEnd )
		if( Map(TileX, TileY) > 0 )	then return true	   '' Found a tile

		TileY += 1										   '' Next tile downward
		TileYpixels +=TILE_SIZE							   '' Next tile (in pixels) downward
		
	Wend
	
	return false
	
End Function


''*****************************************************************************
'' Collides the player box with the tiles on the y-axis
''*****************************************************************************
function Player.CollideFloors(byval ix as integer, byval iy as integer, byref TileY as integer, Map() as integer ) as integer
	
	dim as integer TileXpixels = ix - (ix mod TILE_SIZE)
	dim as integer TestEnd = ix + wid
	
	TileY = iy\TILE_SIZE
	
	dim as integer TileX = TileXpixels\TILE_SIZE
	
	while (TileXpixels <= TestEnd)
		if (Map(TileX, TileY))	then return true	

		TileX += 1
		TileXpixels +=TILE_SIZE
		
	Wend
	
	return false
	
End Function

''*************************************
'' Checks player collisions on the map
''*************************************
function Player.CollideOnMap( Map() as integer ) as integer
	
	dim as Integer TileX, TileY
	dim as integer CollisionType = COLLIDE_NONE    '' Return value. Assume no collision
	
	if( Dx > 0 ) then 		'' Right movement
		
		if( CollideWalls( x + Dx + Wid, y, TileX, Map() ) ) then    '' (x + Dx + wid) = Right side of player
			x = TileX * TILE_SIZE - Wid - 1							'' Snap left when there's a collision
			CollisionType = COLLIDE_RIGHT
		else
			x += Dx													'' No collision, so move
		EndIf
	
	elseif( Dx < 0 ) then 	'' Left movement																					
		
		if( CollideWalls( x + Dx, y, TileX, Map() ) ) then			'' (x + Dx) = Left side of player
			x = ( TileX + 1 ) * TILE_SIZE + 1						'' Snap to right of tile
			CollisionType = COLLIDE_LEFT
		else
			x += Dx													'' No collision, so move
		EndIf
		
	EndIf
	
	
	if( Dy < 0 ) then   	'' moving Up
		
		if( CollideFloors( x, y + Dy, TileY, Map() ) ) then   		'' hit the roof
			y = ( TileY + 1 ) * TILE_SIZE + 1						'' Snap below the tile
			Dy = 0    												'' Arrest movement
			CollisionType = COLLIDE_CEILING
		else
			y += Dy													'' No collision so move
			Dy += GRAVITY											'' with gravity
		EndIf
			
	else	'' Stationary or moving down
		
		if( CollideFloors( x, y + Dy + Hei, TileY, Map() ) ) then	'' (y + Dy + hei) = Foot of player
			y = ( TileY ) * TILE_SIZE - Hei - 1						'' Snap above the tile
			Dy = 1													'' Set to 1 so that we always collide with floor next frame
			CollisionType = COLLIDE_FLOOR
		else
			y += Dy													'' No collision so move
			Dy += GRAVITY
		EndIf
		
	EndIf
	
	return CollisionType
	
End function

''*****************************************************************************
'' Updates the player according to states
''*****************************************************************************
sub Player.Update( Map() as integer )

	Counter += 1
	Animate()
	
	'' Check to see the Player State and
	'' Update accordingly
	
	'' In my DS game I used function pointers 
	'' But I have no idea how to get and call
	'' function pointers of class methods in FB
	'' so I used select case which is not too bad.
	Select Case State
		case IDLE:
			UpdateIdle( Map() )
		case WALKING:
			StandingCounter = 0    '' Set bored counter to zero since we don't want to get bored while walking
			UpdateWalking( Map() )
		case JUMPING:
			StandingCounter = 0
			UpdateJumping( Map() )
		case FALLING:
			StandingCounter = 0
			UpdateFalling( Map() )
		case BORED:
			StandingCounter = 0
			UpdateBored( Map() )
		case LIGHT_DYNAMITE:
			StandingCounter = 0
			UpdateLightDynamite( Map() )
		case PLANT_BOMB:
			StandingCounter = 0
			UpdatePlantBomb( Map() )
		case DIED:
			StandingCounter = 0
			UpdateDied( Map() )
	End Select
    
    
End Sub

''*****************************************************************************
'' Called when player is not moving and standing on the ground
''*****************************************************************************
Sub Player.UpdateIdle( Map() as integer )
	
	Dx = 0  '' Set speed to 0
	
	'' Did we dilly-dally too long?
	'' If so get "bored"
	StandingCounter += 1
	if( StandingCounter > BORED_WAIT_TIME ) then
		State = BORED
		ResolveAnimationParameters()
	EndIf
	
	'' If we pressed left then we walk negatively
	'' and we set the State to WALKING since we moved
	if multikey(FB.SC_LEFT) then 
		Dx = -speed
		State = WALKING
		Direction = DIR_LEFT
		ResolveAnimationParameters()
	EndIf
	
	'' See comments above
    if multikey(FB.SC_RIGHT) then 
    	Dx = speed
    	State = WALKING
    	Direction = DIR_RIGHT
		ResolveAnimationParameters()
    EndIf
    
    '' We can Light a dynamite while standing
    if multikey(FB.SC_Z) then 
    	State = LIGHT_DYNAMITE
		ResolveAnimationParameters()
    EndIf
    
    '' We can plant bombs while standing
    if multikey(FB.SC_X) then 
    	State = PLANT_BOMB
		ResolveAnimationParameters()
    EndIf
    
    '' We can die while standing
    if multikey(FB.SC_C) then 
    	State = DIED
    	Dy = -JUMPHEIGHT	'' Mario Style death		
		ResolveAnimationParameters()
    EndIf
   
    '' We can jump while not moving so jump when we press space
    '' Then set the state to JUMPING
    if multikey(FB.SC_SPACE) then 
	    if( CanJump ) then
	    	State = JUMPING 		
	    	Dy = -JUMPHEIGHT		'' This makes us jump
	    	CanJump = false			'' We can't jump again while in the middle of Jumping
	    	ResolveAnimationParameters()
	    end if
    EndIf
    
	
	'' If there is a collision handle it	
	dim as Integer Collision = CollideOnMap( Map() )
	if( Collision ) then
		select case Collision
			case COLLIDE_FLOOR:		'' Floor so we can jump again
				if( not multikey(FB.SC_SPACE) )  then CanJump = true '' We hit floor so we can jump again
			case else
		End Select
	else											'' No collision so we are on air
		CanJump = false								'' We can't jump since we are..
		if( Dy > GRAVITY * 2 ) then State = FALLING	'' Falling ( Only fall when we after a certain threshold
    	ResolveAnimationParameters()
	EndIf
	
	
End Sub

''*****************************************************************************
'' Called when player is walking on the ground
''*****************************************************************************
Sub Player.UpdateWalking( Map() as integer )
	
	'' Assume we are not moving so set
	'' State to be idle
	State = IDLE
	
	Dx = 0  '' Set speed to 0
	
	'' If we pressed left then we walk negatively
	'' and we set the State to WALKING since we moved
	if multikey(FB.SC_LEFT) then 
		Dx = -speed
		State = WALKING
		Direction = DIR_LEFT
	EndIf
	
	'' See comments above
    if multikey(FB.SC_RIGHT) then 
    	Dx = speed
    	State = WALKING
    	Direction = DIR_RIGHT
    EndIf
    
    '' We can Light a dynamite while walking
    if multikey(FB.SC_Z) then 
    	State = LIGHT_DYNAMITE
		ResolveAnimationParameters()
    EndIf
    
    '' We can plant bombs while walking
    if multikey(FB.SC_X) then 
    	State = PLANT_BOMB
		ResolveAnimationParameters()
    EndIf
    
    '' We can die while walking
    if multikey(FB.SC_C) then 
    	State = DIED
    	Dy = -JUMPHEIGHT	'' Mario Style death		
		ResolveAnimationParameters()
    EndIf
   
    
    '' We can jump while not moving so jump when we press space
    '' Then set the state to JUMPING
    if multikey(FB.SC_SPACE) then 
	    if( CanJump ) then
	    	State = JUMPING 		
	    	Dy = -JUMPHEIGHT		'' This makes us jump
	    	CanJump = false			'' We can't jump while Jumping
	    	ResolveAnimationParameters()
	    end if
    EndIf
    
	'' If there is a collision handle it	
	dim as Integer Collision = CollideOnMap( Map() )
	if( Collision ) then
		select case Collision
			case COLLIDE_FLOOR:		'' Floor so we can jump again
				if( not multikey(FB.SC_SPACE) )  then CanJump = true '' We hit floor so we can jump again
			case else
		End Select
	else											'' No collision so we are on air
		CanJump = false								'' We can't jump since we are..
		if( Dy > GRAVITY * 2 ) then State = FALLING	'' Falling ( Only fall when we after a certain threshold
    	ResolveAnimationParameters()
	EndIf
	
		
	if( State = IDLE ) then ResolveAnimationParameters()
	
End Sub

''*****************************************************************************
'' Called when player is jumping
''*****************************************************************************
Sub Player.UpdateJumping( Map() as integer )
	
	'' You will notice that there is no way to plant bombs or dynamite within this sub
	'' This is the beauty of FSM. You can limit behaviors depending on your needs.
	'' I didn't want the player to plant bombs or dynamites while jumping or falling so
	'' I just didn't include a check here.
	
	dim as integer Walked = FALSE    '' a way to check if we moved left or right
									 '' Since Dx is single and EPSILON would not look
									 '' good in a tutorial
	
	Dx = 0  '' Set speed to 0
	'' We can move left or right when jumping so...
	
	'' If we pressed left then we walk negatively
	'' and we set the State to WALKING since we moved
	if multikey(FB.SC_LEFT) then 
		Dx = -speed
		Walked = TRUE  			''Set walked to true for checking later 
		Direction = DIR_LEFT
	EndIf
	
	'' See comments above
    if multikey(FB.SC_RIGHT) then 
    	Dx = speed
    	Walked = TRUE
    	Direction = DIR_RIGHT
    EndIf
    
    '' Stop jumping when player stops pressing jump key
    if( not multikey(FB.SC_SPACE) ) then
    	if( Dy < 0 ) then Dy = 0
    EndIf
	
    '' We can die while jumping
    if multikey(FB.SC_C) then 
    	State = DIED
    	Dy = -JUMPHEIGHT	'' Mario Style death		
		ResolveAnimationParameters()
    EndIf
   
    
	
	'' If there is a collision handle it	
	dim as Integer Collision = CollideOnMap( Map() )
	if( Collision = COLLIDE_FLOOR ) then
		
		CanJump = FALSE	'' Set this to FALSE since we cannot jump while jumping	
	
		'' Check if we walked or not when we collided with the floor
		if( Walked ) then
			State = WALKING	'' Set the State to WALKING when we hit the floor
			ResolveAnimationParameters()
		else
			State = IDLE	'' Ditto
			ResolveAnimationParameters()
		End If
		
	End If
	
	
End Sub

''*****************************************************************************
'' Called when player is falling
''*****************************************************************************
Sub Player.UpdateFalling( Map() as integer )
	
	'' You will notice that there is no way to plant bombs or dynamite within this sub
	'' This is the beauty of FSM. You can limit behaviors depending on your needs.
	'' I didn't want the player to plant bombs or dynamites while jumping or falling so
	'' I just didn't include a check here.
	
	dim as integer Walked = FALSE    '' a way to check if we moved left or right
									 '' Since Dx is single and EPSILON would not look
									 '' good in a tutorial
	
	Dx = 0  '' Set speed to 0
	'' We can move left or right when falling so...
	
	'' If we pressed left then we walk negatively
	'' and we set the State to WALKING since we moved
	if multikey(FB.SC_LEFT) then 
		Dx = -speed
		Walked = TRUE  			''Set walked to true for checking later 
		Direction = DIR_LEFT
	EndIf
	
	'' See comments above
    if multikey(FB.SC_RIGHT) then 
    	Dx = speed
    	Walked = TRUE
    	Direction = DIR_RIGHT
    EndIf
    
    '' We can die while falling
    if multikey(FB.SC_C) then 
    	State = DIED
    	Dy = -JUMPHEIGHT	'' Mario Style death		
		ResolveAnimationParameters()
    EndIf
   
   
		'' If there is a collision handle it	
	dim as Integer Collision = CollideOnMap( Map() )
	if( Collision = COLLIDE_FLOOR ) then
		
		CanJump = FALSE	'' Set this to FALSE since we cannot jump while jumping	
	
		'' Check if we walked or not
		if( Walked ) then
			State = WALKING	'' Set the State to WALKING when we hit the floor
			ResolveAnimationParameters()
		else
			State = IDLE	'' Ditto
			ResolveAnimationParameters()
		End If
	                
	End If
	
	
End Sub

''*****************************************************************************
'' Called when Player gets bored standing up for a while
''*****************************************************************************
Sub Player.UpdateBored( Map() as integer )
	
	Dx = 0  '' Set speed to 0
	
	'' If we pressed left then we walk negatively
	'' and we set the State to WALKING since we moved
	if multikey(FB.SC_LEFT) then 
		Dx = -speed
		State = WALKING
		Direction = DIR_LEFT
		ResolveAnimationParameters()
	EndIf
	
	'' See comments above
    if multikey(FB.SC_RIGHT) then 
    	Dx = speed
    	State = WALKING
    	Direction = DIR_RIGHT
		ResolveAnimationParameters()
    EndIf
    
    '' We can Light a dynamite while bored
    if multikey(FB.SC_Z) then 
    	State = LIGHT_DYNAMITE
		ResolveAnimationParameters()
    EndIf
    
    '' We can plant bombs while bored
    if multikey(FB.SC_X) then 
    	State = PLANT_BOMB
		ResolveAnimationParameters()
    EndIf
    
    '' We can die while being bored
    if multikey(FB.SC_C) then 
    	State = DIED
    	Dy = -JUMPHEIGHT	'' Mario Style death		
		ResolveAnimationParameters()
    EndIf
   
    '' We can jump while not moving so jump when we press space
    '' Then set the state to JUMPING
    if multikey(FB.SC_SPACE) then 
	    if( CanJump ) then
	    	State = JUMPING 		
	    	Dy = -JUMPHEIGHT		'' This makes us jump
	    	CanJump = false			'' We can't jump while Jumping
	    	ResolveAnimationParameters()
	    end if
    EndIf
    

	dim as Integer Collision = CollideOnMap( Map() )
	if( Collision = COLLIDE_FLOOR ) then
		CanJump = true		'' We hit floor so we can jump again	
	EndIf
	
End Sub


''*****************************************************************************
'' Called when Player lights up a dynamite
''*****************************************************************************
Sub Player.UpdateLightDynamite( Map() as integer )
	
	
	'' We can die while lighting a dynamite
    if multikey(FB.SC_C) then 
    	State = DIED
    	Dy = -JUMPHEIGHT	'' Mario Style death		
		ResolveAnimationParameters()
    EndIf
   
	'' We don't move while lighting up a dynamite
	'' We can only light a dynamite if we are either
	'' STANDING, WALKING, BORED
	'' We can't light a Dynamite while Jumping
	Dx = 0  '' Set speed to 0
	Dy = 0	
	if( Frame = (MaxFrame-1) ) then
		State = IDLE
		ResolveAnimationParameters()
	EndIf
	
End Sub

''*****************************************************************************
'' Called when Player lights up a dynamite
''*****************************************************************************
Sub Player.UpdatePlantBomb( Map() as integer )
	
	'' We can die while planting bombs
    if multikey(FB.SC_C) then 
    	State = DIED
    	Dy = -JUMPHEIGHT	'' Mario Style death		
		ResolveAnimationParameters()
    EndIf
   
   '' We don't move while Planting Bombs
	'' We can only Plant bombs if we are either
	'' STANDING, WALKING, BORED
	'' We can't Plant bombs while Jumping
	Dx = 0  '' Set speed to 0
	Dy = 0	
	if( Frame = (MaxFrame-1) ) then
		State = IDLE
		ResolveAnimationParameters()
	EndIf
	
End Sub


''*****************************************************************************
'' Called when player is dead
''*****************************************************************************
Sub Player.UpdateDied( Map() as integer )

	'' We can't do anything except fall down when dying so no checks for
	'' walking, jumping, dynamite, bombs, etc.
	Dx = 0
	dim as Integer Collision = CollideOnMap( Map() )
	if( Collision = COLLIDE_FLOOR ) then
		State = IDLE
		ResolveAnimationParameters()	
	EndIf
	
	
End Sub


''*****************************************************************************
'' Draws the player according to state
''*****************************************************************************
sub Player.Draw(SpriteSet() as GL2D.IMAGE ptr)

		
	Select Case State
		case IDLE:
			GL2D.PrintScale(0, 0, 1, "State = IDLE" )
		case WALKING:
			GL2D.PrintScale(0, 0, 1, "State = WALKING" )
		case JUMPING:
			GL2D.PrintScale(0, 0, 1, "State = JUMPING" )
		case FALLING:
			GL2D.PrintScale(0, 0, 1, "State = FALLING" )
		case BORED:
			GL2D.PrintScale(0, 0, 1, "State = BORED" )
		case LIGHT_DYNAMITE:
			GL2D.PrintScale(0, 0, 1, "State = LIGHT_DYNAMITE" )
		case PLANT_BOMB:
			GL2D.PrintScale(0, 0, 1, "State = PLANT_BOMB" )
		case DIED:
			GL2D.PrintScale(0, 0, 1, "State = DIED" )
	End Select
		
		
				
		GL2D.PrintScale(0, 20, 1, "Bored counter = " & str(StandingCounter) )    
	
		GL2D.PrintScale(0, 30, 1, "Bored wait = " & str(BORED_WAIT_TIME) )    
	
		GL2D.PrintScale(0, 20, 1, "Bored counter = " & str(StandingCounter) )    
	
		GL2D.PrintScale(0, 30, 1, "Bored wait = " & str(BORED_WAIT_TIME) )    
		
		GL2D.PrintScale(0,  50, 1, "CONTROLS")    
		GL2D.PrintScale(0,  70, 1, "ARROWS = Move")    
		GL2D.PrintScale(0,  80, 1, " SPACE = Jump")    
		GL2D.PrintScale(0,  90, 1, "     Z = Light Dynamite")    
		GL2D.PrintScale(0, 100, 1, "     X = Plant Bomb")
		GL2D.PrintScale(0, 110, 1, "     C = Die")    
	    
		GL2D.SpriteFlip( x-4, y, FlipMode, SpriteSet(BaseFrame + Frame))
	
end sub

''*****************************************************************************
'' Animates the player
''*****************************************************************************
Sub Player.Animate()
	
	If( (Counter and 3) = 0 ) Then
		Frame = ( Frame + 1 ) mod MaxFrame
	EndIf
	
	if( Direction = DIR_RIGHT ) then
		FlipMode = GL2D.FLIP_NONE
	else
		FlipMode = GL2D.FLIP_H
	EndIf
	
End Sub


''*****************************************************************************
'' Sets up animation frames depending on current state
''*****************************************************************************
sub Player.ResolveAnimationParameters()
	
	Select Case State
		case IDLE:
			Frame = 0
			BaseFrame = 16
			MaxFrame = 1	
		case WALKING:
			Frame = 0
			BaseFrame = 0
			MaxFrame = 8
		case JUMPING:
			Frame = 0
			BaseFrame = 17
			MaxFrame = 1
		case FALLING:
			Frame = 0
			BaseFrame = 17
			MaxFrame = 1
		case BORED:
			Frame = 0
			BaseFrame = 9
			MaxFrame = 7
		case LIGHT_DYNAMITE:
			Frame = 0
			BaseFrame = 18
			MaxFrame = 7   '' offset by 1 so that all the frames get drawn
						   '' for we will change states after animation is done
		case PLANT_BOMB:
			Frame = 0
			BaseFrame = 24
			MaxFrame = 5   '' offset by 1 so that all the frames get drawn
						   '' for we will change states after animation is done
		case DIED:
			Frame = 0
			BaseFrame = 28
			MaxFrame = 4   
		End Select
    
End Sub


property Player.GetX() as Single
	Property = x
End Property

property Player.GetY() as Single
	Property = y
End Property

''*****************************************************************************
'' Our main sub
''*****************************************************************************
sub main()

	
	'' Our map for drawing and collision
	'' On a real tilebased engine, it is better to use another
	'' map for collision aka "collision maps"
	dim as integer Map()
	
	'' Temporary string array for easy map making
	'' See ConvertMap() and "Tiles.BMP" for more details
	'' @ = Tile 1 
	'' # = Tile 2
	'' () = Tile 3 and 4 
	dim as String TempMap(20) =>_
	{	"####################",_  		'' 0
		"#                  #",_		'' 1
		"#                  #",_		'' 2
		"#           ########",_		'' 3
		"#         #        #",_		'' 4
		"########           #",_		'' 5
		"#       #     ## ###",_		'' 6
		"#                  #",_		'' 7
		"#       ####       #",_		'' 8
		"#             #    #",_		'' 9
		"#     ##           #",_		'' 0
		"#        # #####   #",_		'' 1
		"# ###    ##     ## #",_		'' 2
		"#                  #",_		'' 3
		"####################"	}		'' 4   
	    '01234567890123456789

	'' Convert the Ascii map to a 2D integer map
	ConvertMap( Map(), TempMap() )

	
	redim as GL2D.IMAGE ptr GripeImages(0)
	
	
	dim as integer Frame = 0
	 
	dim as Player Gripe
	 
	GL2D.ScreenInit( SCREEN_WIDTH, SCREEN_HEIGHT )   ''Set up GL screen
	GL2D.VsyncON()
	
	GL2D.InitSprites( GripeImages(), 32,32, "gripe.bmp", GL_NEAREST )

	do
		
		Frame += 1

		Gripe.Update( Map() )     '' Do player movements
    
		GL2D.ClearScreen()
			
		GL2D.Begin2D()
			
			'' Draw map
			DrawMap( Gripe.GetX, Gripe.GetY, Map() )
			
			'' Draw Player
			Gripe.Draw( GripeImages() )
			
			GL2D.PrintScale(0,  140, 1, "Entity management using FSM")    
			GL2D.PrintScale(0,  150, 1, "Code by Relminator")    
			GL2D.PrintScale(0,  160, 1, "Sprites made by Marc Russel (SpicyPixel.Net)")    
		
			
		GL2D.End2D()
		
		
		dim as integer FPS = GL2D.LimitFPS(60)
		
		sleep 1,1
		flip
		
	Loop until multikey(FB.SC_ESCAPE)

	GL2D.DestroySprites( GripeImages() )

	GL2D.ShutDown()
	
End Sub


''*************************************
'' Converts an Ascii map to integer map 
''*************************************
sub ConvertMap( Map() as integer, StrMap() as string )
	
	'' Resize array according to size of ascii map
	redim Map(len(Strmap(0))-1, ubound(StrMap) )
	
	'' @ = Tile 1 
	'' # = Tile 2
	'' () = Tile 3 and 4
	'' Only used 4 tiles
	 
	for y as integer = 0 to ubound(StrMap)	
		for x as integer = 1 to Len(StrMap(y))
			dim as string a = Mid(StrMap(y), x, 1)
			select case a
				case "#"
					Map(x-1,y) = 1
			End Select
		Next
	Next

	
End Sub



''*************************************
'' Draws the map in 3D
'' Only draws what can be seen so this
'' is fast
''*************************************
sub DrawMap( byval PlayerX as single, byval PlayerY as single, Map() as integer )
	
	'' map dimensions
	dim as integer MAP_WID = Ubound(Map,1)
	dim as integer MAP_HEI = Ubound(Map,2)
	
	'' Read Tile values on the 2D array
	'' Then draw if not empty(0)
	for y as integer = 0 to MAP_HEI
		for x as integer = 0 to MAP_WID
			dim as integer Tile = Map(x,y)
			if( Tile > 0 ) then
				GL2D.BoxFilled( x * TILE_SIZE,_ 
								y * TILE_SIZE,_ 
								x * TILE_SIZE + TILE_SIZE,_ 
								y * TILE_SIZE + TILE_SIZE,_ 
								GL2D_RGBA(255,128,120,255) )
			end if
		next x	
	next y
	

	
end sub



''*****************************************************************************
''
''
''
''*****************************************************************************


main()


end




